#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import polyaxon_sdk

from marshmallow import EXCLUDE, fields

from polyaxon.api import get_default_host
from polyaxon.containers.contexts import CONTEXT_ARCHIVE_ROOT
from polyaxon.env_vars.keys import (
    POLYAXON_KEYS_API_VERSION,
    POLYAXON_KEYS_ARCHIVE_ROOT,
    POLYAXON_KEYS_ASSERT_HOSTNAME,
    POLYAXON_KEYS_AUTHENTICATION_TYPE,
    POLYAXON_KEYS_CERT_FILE,
    POLYAXON_KEYS_CONNECTION_POOL_MAXSIZE,
    POLYAXON_KEYS_DEBUG,
    POLYAXON_KEYS_DISABLE_ERRORS_REPORTING,
    POLYAXON_KEYS_HEADER,
    POLYAXON_KEYS_HEADER_SERVICE,
    POLYAXON_KEYS_HOST,
    POLYAXON_KEYS_INTERVAL,
    POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK,
    POLYAXON_KEYS_IS_MANAGED,
    POLYAXON_KEYS_IS_OFFLINE,
    POLYAXON_KEYS_IS_OPS,
    POLYAXON_KEYS_K8S_IN_CLUSTER,
    POLYAXON_KEYS_K8S_NAMESPACE,
    POLYAXON_KEYS_KEY_FILE,
    POLYAXON_KEYS_LOG_LEVEL,
    POLYAXON_KEYS_NO_API,
    POLYAXON_KEYS_NO_OP,
    POLYAXON_KEYS_SERVICE,
    POLYAXON_KEYS_SSL_CA_CERT,
    POLYAXON_KEYS_TIME_ZONE,
    POLYAXON_KEYS_TIMEOUT,
    POLYAXON_KEYS_TRACKING_TIMEOUT,
    POLYAXON_KEYS_VERIFY_SSL,
    POLYAXON_KEYS_WATCH_INTERVAL,
)
from polyaxon.exceptions import PolyaxonClientException
from polyaxon.pkg import VERSION
from polyaxon.schemas.base import BaseConfig, BaseSchema
from polyaxon.services.auth import AuthenticationTypes
from polyaxon.services.headers import PolyaxonServiceHeaders, PolyaxonServices
from polyaxon.utils.http_utils import clean_host, clean_verify_ssl


class ClientSchema(BaseSchema):
    service = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_SERVICE)
    host = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_HOST)
    version = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_API_VERSION)
    debug = fields.Bool(allow_none=True, data_key=POLYAXON_KEYS_DEBUG)
    log_level = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_LOG_LEVEL)
    authentication_type = fields.Str(
        allow_none=True, data_key=POLYAXON_KEYS_AUTHENTICATION_TYPE
    )
    is_managed = fields.Bool(allow_none=True, data_key=POLYAXON_KEYS_IS_MANAGED)
    is_offline = fields.Bool(allow_none=True, data_key=POLYAXON_KEYS_IS_OFFLINE)
    is_ops = fields.Bool(allow_none=True, data_key=POLYAXON_KEYS_IS_OPS)
    in_cluster = fields.Bool(allow_none=True, data_key=POLYAXON_KEYS_K8S_IN_CLUSTER)
    no_op = fields.Bool(allow_none=True, data_key=POLYAXON_KEYS_NO_OP)
    timeout = fields.Float(allow_none=True, data_key=POLYAXON_KEYS_TIMEOUT)
    tracking_timeout = fields.Float(
        allow_none=True, data_key=POLYAXON_KEYS_TRACKING_TIMEOUT
    )
    timezone = fields.Str(
        allow_none=True, data_key=POLYAXON_KEYS_TIME_ZONE, default="UTC"
    )
    watch_interval = fields.Int(allow_none=True, data_key=POLYAXON_KEYS_WATCH_INTERVAL)
    interval = fields.Float(allow_none=True, data_key=POLYAXON_KEYS_INTERVAL)
    verify_ssl = fields.Bool(allow_none=True, data_key=POLYAXON_KEYS_VERIFY_SSL)
    ssl_ca_cert = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_SSL_CA_CERT)
    cert_file = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_CERT_FILE)
    key_file = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_KEY_FILE)
    assert_hostname = fields.Bool(
        allow_none=True, data_key=POLYAXON_KEYS_ASSERT_HOSTNAME
    )
    connection_pool_maxsize = fields.Int(
        allow_none=True, data_key=POLYAXON_KEYS_CONNECTION_POOL_MAXSIZE
    )
    archive_root = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_ARCHIVE_ROOT)

    header = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_HEADER)
    header_service = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_HEADER_SERVICE)

    namespace = fields.Str(allow_none=True, data_key=POLYAXON_KEYS_K8S_NAMESPACE)
    no_api = fields.Bool(allow_none=True, data_key=POLYAXON_KEYS_NO_API)
    disable_errors_reporting = fields.Bool(
        allow_none=True, data_key=POLYAXON_KEYS_DISABLE_ERRORS_REPORTING
    )
    compatibility_check_interval = fields.Int(
        allow_none=True, data_key=POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK
    )

    @staticmethod
    def schema_config():
        return ClientConfig


class ClientConfig(BaseConfig):
    SCHEMA = ClientSchema
    IDENTIFIER = "global"

    PAGE_SIZE = 20
    BASE_URL = "{}/api/{}"

    UNKNOWN_BEHAVIOUR = EXCLUDE

    REDUCED_ATTRIBUTES = [
        POLYAXON_KEYS_SERVICE,
        POLYAXON_KEYS_HOST,
        POLYAXON_KEYS_API_VERSION,
        POLYAXON_KEYS_ASSERT_HOSTNAME,
        POLYAXON_KEYS_AUTHENTICATION_TYPE,
        POLYAXON_KEYS_CERT_FILE,
        POLYAXON_KEYS_CONNECTION_POOL_MAXSIZE,
        POLYAXON_KEYS_ARCHIVE_ROOT,
        POLYAXON_KEYS_DEBUG,
        POLYAXON_KEYS_HEADER,
        POLYAXON_KEYS_HEADER_SERVICE,
        POLYAXON_KEYS_K8S_IN_CLUSTER,
        POLYAXON_KEYS_INTERVAL,
        POLYAXON_KEYS_IS_MANAGED,
        POLYAXON_KEYS_IS_OFFLINE,
        POLYAXON_KEYS_IS_OPS,
        POLYAXON_KEYS_K8S_NAMESPACE,
        POLYAXON_KEYS_KEY_FILE,
        POLYAXON_KEYS_LOG_LEVEL,
        POLYAXON_KEYS_NO_API,
        POLYAXON_KEYS_NO_OP,
        POLYAXON_KEYS_SSL_CA_CERT,
        POLYAXON_KEYS_TIMEOUT,
        POLYAXON_KEYS_TRACKING_TIMEOUT,
        POLYAXON_KEYS_VERIFY_SSL,
        POLYAXON_KEYS_WATCH_INTERVAL,
        POLYAXON_KEYS_DISABLE_ERRORS_REPORTING,
        POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK,
    ]

    def __init__(
        self,
        service=None,
        host=None,
        token=None,
        debug=None,
        log_level=None,
        version=None,
        authentication_type=None,
        is_managed=None,
        is_offline=None,
        is_ops=None,
        in_cluster=None,
        no_op=None,
        timeout=None,
        tracking_timeout=None,
        timezone=None,
        watch_interval=None,
        interval=None,
        verify_ssl=None,
        ssl_ca_cert=None,
        cert_file=None,
        key_file=None,
        assert_hostname=None,
        connection_pool_maxsize=None,
        archive_root=None,
        header=None,
        header_service=None,
        namespace=None,
        no_api=None,
        disable_errors_reporting=None,
        compatibility_check_interval=None,
        **kwargs
    ):
        self.service = service
        self.host = clean_host(get_default_host(host, service))
        self.token = token
        self.debug = self._get_bool(debug, False)
        self.log_level = log_level
        self.version = version or "v1"
        self.is_managed = self._get_bool(is_managed, False)
        self.is_offline = self._get_bool(is_offline, False)
        self.is_ops = self._get_bool(is_ops, False)
        self.in_cluster = self._get_bool(in_cluster, False)
        self.no_op = self._get_bool(no_op, False)
        self.verify_ssl = clean_verify_ssl(
            host=self.host, verify_ssl=self._get_bool(verify_ssl, None)
        )
        self.ssl_ca_cert = ssl_ca_cert
        self.cert_file = cert_file
        self.key_file = key_file
        self.assert_hostname = self._get_bool(assert_hostname, None)
        self.connection_pool_maxsize = connection_pool_maxsize
        self.archive_root = archive_root or CONTEXT_ARCHIVE_ROOT
        self.header = header
        self.header_service = header_service
        self.timeout = timeout or 20
        self.tracking_timeout = tracking_timeout or 1
        self.timezone = timezone
        self.interval = interval or 5
        self.watch_interval = watch_interval or 5
        self.namespace = namespace
        self.no_api = self._get_bool(no_api, False)
        self.authentication_type = authentication_type or AuthenticationTypes.TOKEN
        self.disable_errors_reporting = self._get_bool(disable_errors_reporting, False)
        self.compatibility_check_interval = compatibility_check_interval

        self.client_header = {}

        if all([self.header, self.header_service]):
            self.client_header["header_name"] = self.header
            self.client_header["header_value"] = self.header_service

    @property
    def base_url(self):
        return self.BASE_URL.format(clean_host(self.host), self.version)

    def set_cli_header(self):
        self.header = PolyaxonServiceHeaders.get_header(PolyaxonServiceHeaders.SERVICE)
        self.header_service = VERSION
        self.client_header["header_name"] = self.header
        self.client_header["header_value"] = self.header_service

    def set_agent_header(self):
        self.header = PolyaxonServiceHeaders.get_header(PolyaxonServiceHeaders.SERVICE)
        self.header_service = PolyaxonServices.AGENT
        self.client_header["header_name"] = self.header
        self.client_header["header_value"] = self.header_service

    @property
    def sdk_config(self):
        if not self.host and not self.in_cluster:
            raise PolyaxonClientException(
                "Api config requires at least a host if not running in-cluster."
            )

        config = polyaxon_sdk.Configuration()
        config.debug = self.debug
        config.host = clean_host(self.host)
        config.verify_ssl = clean_verify_ssl(
            host=config.host, verify_ssl=self.verify_ssl
        )
        config.ssl_ca_cert = self.ssl_ca_cert
        config.cert_file = self.cert_file
        config.key_file = self.key_file
        config.assert_hostname = self.assert_hostname
        if self.connection_pool_maxsize:
            config.connection_pool_maxsize = self.connection_pool_maxsize
        if self.token:
            config.api_key["Authorization"] = self.token
            config.api_key_prefix["Authorization"] = self.authentication_type
        return config

    @staticmethod
    def _get_bool(value, default_value):
        if isinstance(value, bool):
            return value

        return default_value



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon import settings
from polyaxon.client.transport.retry_transport import RetryTransportMixin
from polyaxon.client.workers.queue_worker import QueueWorker
from polyaxon.logger import logger


class ThreadedTransportMixin(RetryTransportMixin):
    """Threads operations transport."""

    @property
    def threaded_done(self):
        if hasattr(self, "_threaded_done"):
            return self._threaded_done
        return None

    @property
    def threaded_exceptions(self):
        if hasattr(self, "_threaded_exceptions"):
            return self._threaded_exceptions
        return None

    def queue_request(self, request, url, **kwargs):
        try:
            request(url=url, session=self.retry_session, **kwargs)
        except Exception as e:
            self._threaded_exceptions += 1
            logger.debug(
                "Error making request url: %s, params: params %s, exp: %s",
                url,
                kwargs,
                e,
            )
        finally:
            self._threaded_done += 1

    @property
    def worker(self):
        if not hasattr(self, "_worker") or not self._worker.is_alive():
            self._worker = QueueWorker(timeout=settings.CLIENT_CONFIG.timeout)
            self._worker.start()
        return self._worker

    def async_post(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
    ):
        """Async Call request with a post."""
        return self.worker.queue(
            self.queue_request,
            request=self.post,
            url=url,
            params=params,
            data=data,
            files=files,
            json_data=json_data,
            timeout=timeout,
            headers=headers,
        )

    def async_patch(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
    ):
        """Async Call request with a patch."""
        return self.worker.queue(
            self.queue_request,
            request=self.patch,
            url=url,
            params=params,
            data=data,
            files=files,
            json_data=json_data,
            timeout=timeout,
            headers=headers,
        )

    def async_delete(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
    ):
        """Async Call request with a delete."""
        return self.worker.queue(
            self.queue_request,
            request=self.delete,
            url=url,
            params=params,
            data=data,
            files=files,
            json_data=json_data,
            timeout=timeout,
            headers=headers,
        )

    def async_put(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
    ):
        """Async Call request with a put."""
        return self.worker.queue(
            self.queue_request,
            request=self.put,
            url=url,
            params=params,
            data=data,
            files=files,
            json_data=json_data,
            timeout=timeout,
            headers=headers,
        )

    def async_upload(
        self,
        url,
        files,
        files_size,
        params=None,
        json_data=None,
        timeout=3600,
        headers=None,
    ):
        return self.worker.queue(
            self.queue_request,
            request=self.upload,
            url=url,
            files=files,
            files_size=files_size,
            params=params,
            json_data=json_data,
            timeout=timeout,
            headers=headers,
        )



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import queue
import threading
import time

from typing import Dict, List, Union

from polyaxon.polyboard.events import LoggedEventSpec, get_asset_path, get_event_path
from polyaxon.polyboard.events.paths import get_resource_path
from polyaxon.polyboard.events.schemas import LoggedEventListSpec
from polyaxon.polyboard.processors.gpu_processor import (
    can_log_gpu_resources,
    get_gpu_metrics,
)
from polyaxon.polyboard.processors.psutil_processor import (
    can_log_psutil_resources,
    get_psutils_metrics,
)
from polyaxon.utils.path_utils import check_or_create_path


class EventWriter:
    EVENTS_BACKEND = "events"
    RESOURCES_BACKEND = "resources"

    def __init__(self, run_path: str, backend: str):
        self._events_backend = backend
        self._run_path = run_path
        self._files = {}  # type: Dict[str, LoggedEventListSpec]
        self._closed = False

    def _get_event_path(self, kind: str, name: str) -> str:
        if self._events_backend == self.EVENTS_BACKEND:
            return os.path.join(
                self._run_path, self._events_backend, kind, "{}.plx".format(name)
            )
        if self._events_backend == self.RESOURCES_BACKEND:
            return os.path.join(
                self._run_path, self._events_backend, kind, "{}.plx".format(name)
            )
        raise ValueError("Unrecognized backend {}".format(self._events_backend))

    def _init_events(self, events_spec: LoggedEventListSpec):
        event_path = self._get_event_path(kind=events_spec.kind, name=events_spec.name)
        # Check if the file exists otherwise initialize
        if not os.path.exists(event_path):
            check_or_create_path(event_path, is_dir=False)
            with open(event_path, "w") as event_file:
                event_file.write(events_spec.get_csv_header())

    def _append_events(self, events_spec: LoggedEventListSpec):
        event_path = self._get_event_path(kind=events_spec.kind, name=events_spec.name)
        with open(event_path, "a") as event_file:
            event_file.write(events_spec.get_csv_events())

    def _events_to_files(self, events: List[LoggedEventSpec]):
        for event in events:
            file_name = "{}.{}".format(event.kind, event.name)
            if file_name in self._files:
                self._files[file_name].events.append(event.event)
            else:
                self._files[file_name] = LoggedEventListSpec(
                    kind=event.kind, name=event.name, events=[event.event]
                )
                self._init_events(self._files[file_name])

    def write(self, events: List[LoggedEventSpec]):
        if not events:
            return
        if isinstance(events, LoggedEventSpec):
            events = [events]
        self._events_to_files(events)

    def flush(self):
        for file_name in self._files:
            events_spec = self._files[file_name]
            if events_spec.events:
                self._append_events(events_spec)
            self._files[file_name].empty_events()

    def close(self):
        self.flush()
        self._closed = True

    @property
    def closed(self):
        return self._closed


class BaseFileWriter:
    """Writes `LoggedEventSpec` to event files.

    The `EventFileWriter` class creates a event files in the run path,
    and asynchronously writes Events to the files.
    """

    def __init__(self, run_path: str):
        self._run_path = run_path
        check_or_create_path(run_path, is_dir=True)

    @property
    def run_path(self):
        return self._run_path

    def add_event(self, event: LoggedEventSpec):
        if not isinstance(event, LoggedEventSpec):
            raise TypeError("Expected an LoggedEventSpec, " " but got %s" % type(event))
        self._async_writer.write(event)

    def add_events(self, events: List[LoggedEventSpec]):
        for e in events:
            if not isinstance(e, LoggedEventSpec):
                raise TypeError("Expected an LoggedEventSpec, " " but got %s" % type(e))
        self._async_writer.write(events)

    def flush(self):
        """Flushes the event files to disk.

        Call this method to make sure that all pending events have been
        written to disk.
        """
        self._async_writer.flush()

    def close(self):
        """Performs a final flush of the event files to disk, stops the
        write/flush worker and closes the files.

        Call this method when you do not need the writer anymore.
        """
        self._async_writer.close()


class EventFileWriter(BaseFileWriter):
    def __init__(self, run_path: str, max_queue_size: int = 20, flush_secs: int = 10):
        """Creates a `EventFileWriter`.

        Args:
          run_path: A string. Directory where events files will be written.
          max_queue_size: Integer. Size of the queue for pending events and summaries.
          flush_secs: Number. How often, in seconds, to flush the
            pending events and summaries to disk.
        """
        super().__init__(run_path=run_path)

        check_or_create_path(get_event_path(run_path), is_dir=True)
        check_or_create_path(get_asset_path(run_path), is_dir=True)

        self._async_writer = EventAsyncManager(
            EventWriter(self._run_path, backend=EventWriter.EVENTS_BACKEND),
            max_queue_size,
            flush_secs,
        )


class ResourceFileWriter(BaseFileWriter):
    def __init__(self, run_path: str, max_queue_size: int = 20, flush_secs: int = 10):
        """Creates a `ResourceFileWriter`.

        Args:
          run_path: A string. Directory where events files will be written.
          max_queue_size: Integer. Size of the queue for pending events and summaries.
          flush_secs: Number. How often, in seconds, to flush the
            pending events and summaries to disk.
        """
        super().__init__(run_path=run_path)

        check_or_create_path(get_resource_path(run_path), is_dir=True)

        self._async_writer = ResourceAsyncManager(
            EventWriter(self._run_path, backend=EventWriter.RESOURCES_BACKEND),
            max_queue_size,
            flush_secs,
        )


class BaseAsyncManager:
    """Base manager for writing events to files by name by event kind."""

    def __init__(self, event_writer: EventWriter, max_queue_size: int = 20):
        """Writes events json spec to files asynchronously. An instance of this class
        holds a queue to keep the incoming data temporarily. Data passed to the
        `write` function will be put to the queue and the function returns
        immediately. This class also maintains a thread to write data in the
        queue to disk.

        Args:
            event_writer: A EventWriter instance
            max_queue_size: Integer. Size of the queue for pending bytestrings.
            flush_secs: Number. How often, in seconds, to flush the
                pending bytestrings to disk.
        """
        self._event_writer = event_writer
        self._closed = False
        self._event_queue = queue.Queue(max_queue_size)
        self._lock = threading.Lock()
        self._worker = None

    def write(self, event: Union[LoggedEventSpec, List[LoggedEventSpec]]):
        """Enqueue the given event to be written asychronously."""
        with self._lock:
            if self._closed:
                raise IOError("Writer is closed")
            self._event_queue.put(event)

    def flush(self):
        """Write all the enqueued events before this flush call to disk.

        Block until all the above events are written.
        """
        with self._lock:
            if self._closed:
                raise IOError("Writer is closed")
            self._event_queue.join()
            self._event_writer.flush()

    def close(self):
        """Closes the underlying writer, flushing any pending writes first."""
        if not self._closed:
            with self._lock:
                if not self._closed:
                    self._closed = True
                    self._worker.stop()
                    self._event_writer.flush()
                    self._event_writer.close()


class EventAsyncManager(BaseAsyncManager):
    """Writes events to files by name by event kind."""

    def __init__(
        self, event_writer: EventWriter, max_queue_size: int = 20, flush_secs: int = 10
    ):
        super().__init__(event_writer=event_writer, max_queue_size=max_queue_size)
        self._worker = EventWriterThread(
            self._event_queue, self._event_writer, flush_secs
        )
        self._worker.start()


class EventWriterThread(threading.Thread):
    """Thread that processes asynchronous writes for EventWriter."""

    def __init__(self, event_queue, event_writer: EventWriter, flush_secs: int):
        """Creates an EventWriterThread.

        Args:
          event_queue: A Queue from which to dequeue data.
          event_writer: An instance of EventWriter.
          flush_secs: How often, in seconds, to flush the
            pending file to disk.
        """
        threading.Thread.__init__(self)
        self.daemon = True
        self._event_queue = event_queue
        self._event_writer = event_writer
        self._flush_secs = flush_secs
        # The first data will be flushed immediately.
        self._next_flush_time = 0
        self._has_pending_data = False
        self._shutdown_signal = object()

    def stop(self):
        self._event_queue.put(self._shutdown_signal)
        self.join()

    def run(self):
        # Wait for the queue until data appears, or until the next
        # time to flush the writer.
        # Invoke write If we have data.
        # If not, an empty queue exception will be raised and invoke writer flush.
        while True:
            now = time.time()
            queue_wait_duration = self._next_flush_time - now
            data = None
            try:
                if queue_wait_duration > 0:
                    data = self._event_queue.get(True, queue_wait_duration)
                else:
                    data = self._event_queue.get(False)

                if data is self._shutdown_signal:
                    return
                self._event_writer.write(data)
                self._has_pending_data = True
            except queue.Empty:
                pass
            finally:
                if data:
                    self._event_queue.task_done()

            now = time.time()
            if now > self._next_flush_time:
                if self._has_pending_data:
                    # Small optimization - if there are no pending data,
                    # there's no need to flush.
                    self._event_writer.flush()
                    self._has_pending_data = False
                # Do it again in flush_secs.
                self._next_flush_time = now + self._flush_secs


class ResourceAsyncManager(BaseAsyncManager):
    """Writes resource events to files by name by event kind."""

    def __init__(
        self, event_writer: EventWriter, max_queue_size: int = 20, flush_secs: int = 10
    ):
        super().__init__(event_writer=event_writer, max_queue_size=max_queue_size)
        self._worker = ResourceWriterThread(
            self._event_queue,
            self._event_writer,
            flush_secs,
        )
        self._worker.start()


class ResourceWriterThread(EventWriterThread):
    """Thread that processes periodic resources (cpu, gpu, memory) writes for EventWriter."""

    def __init__(self, event_queue, event_writer: EventWriter, flush_secs: int):
        super().__init__(
            event_queue=event_queue, event_writer=event_writer, flush_secs=flush_secs
        )
        self._log_psutil_resources = can_log_psutil_resources()
        self._log_gpu_resources = can_log_gpu_resources()

    def run(self):
        # Wait for flush time to invoke the writer.
        while True:
            now = time.time()
            queue_wait_duration = self._next_flush_time - now
            data = None
            try:
                if queue_wait_duration > 0:
                    data = self._event_queue.get(True, queue_wait_duration)
                else:
                    data = self._event_queue.get(False)

                if data is self._shutdown_signal:
                    return
                self._event_writer.write(data)
                self._has_pending_data = True
            except queue.Empty:
                pass
            finally:
                if data:
                    self._event_queue.task_done()

            now = time.time()
            if now > self._next_flush_time:
                data = []
                if self._log_psutil_resources:
                    try:
                        data += get_psutils_metrics()
                    except Exception:
                        pass
                try:
                    data += get_gpu_metrics()
                except Exception:
                    pass
                if data:
                    self._event_writer.write(data)
                    self._event_writer.flush()
                self._next_flush_time = now + self._flush_secs



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import time

try:
    import requests
except ImportError:
    raise ImportError("This module depends on requests.")


def safe_request(
    url,
    method=None,
    params=None,
    data=None,
    json=None,
    headers=None,
    allow_redirects=False,
    timeout=30,
    verify_ssl=True,
):
    """A slightly safer version of `request`."""

    session = requests.Session()

    kwargs = {}

    if json:
        kwargs["json"] = json
        if not headers:
            headers = {}
        headers.setdefault("Content-Type", "application/json")

    if data:
        kwargs["data"] = data

    if params:
        kwargs["params"] = params

    if headers:
        kwargs["headers"] = headers

    if method is None:
        method = "POST" if (data or json) else "GET"

    response = session.request(
        method=method,
        url=url,
        allow_redirects=allow_redirects,
        timeout=timeout,
        verify=verify_ssl,
        **kwargs
    )

    return response


STREAM = sys.stderr

BAR_TEMPLATE = "%s[%s%s] %i/%i - %s\r"
MILL_TEMPLATE = "%s %s %i/%i\r"

DOTS_CHAR = "."
BAR_FILLED_CHAR = "#"
BAR_EMPTY_CHAR = " "
MILL_CHARS = ["|", "/", "-", "\\"]

# How long to wait before recalculating the ETA
ETA_INTERVAL = 1
# How many intervals (excluding the current one) to calculate the simple moving average
ETA_SMA_WINDOW = 9


class Bar:
    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.done()
        return False  # we're not suppressing exceptions

    def __init__(
        self,
        label="",
        width=32,
        hide=None,
        empty_char=BAR_EMPTY_CHAR,
        filled_char=BAR_FILLED_CHAR,
        expected_size=None,
        every=1,
    ):
        self.label = label
        self.width = width
        self.hide = hide
        # Only show bar in terminals by default (better for piping, logging etc.)
        if hide is None:
            try:
                self.hide = not STREAM.isatty()
            except AttributeError:  # output does not support isatty()
                self.hide = True
        self.empty_char = empty_char
        self.filled_char = filled_char
        self.expected_size = expected_size
        self.every = every
        self.start = time.time()
        self.ittimes = []
        self.eta = 0
        self.etadelta = time.time()
        self.etadisp = self.format_time(self.eta)
        self.last_progress = 0
        self.elapsed = None
        if self.expected_size:
            self.show(0)

    def show(self, progress, count=None):
        if count is not None:
            self.expected_size = count
        if self.expected_size is None:
            raise Exception("expected_size not initialized")
        self.last_progress = progress
        if (time.time() - self.etadelta) > ETA_INTERVAL:
            self.etadelta = time.time()
            self.ittimes = self.ittimes[-ETA_SMA_WINDOW:] + [
                -(self.start - time.time()) / (progress + 1)
            ]
            self.eta = (
                sum(self.ittimes)
                / float(len(self.ittimes))
                * (self.expected_size - progress)
            )
            self.etadisp = self.format_time(self.eta)
        x = int(self.width * progress / self.expected_size)
        if not self.hide:
            # True every "every" updates and when we're done
            if (progress % self.every) == 0 or (progress == self.expected_size):
                STREAM.write(
                    BAR_TEMPLATE
                    % (
                        self.label,
                        self.filled_char * x,
                        self.empty_char * (self.width - x),
                        progress,
                        self.expected_size,
                        self.etadisp,
                    )
                )
                STREAM.flush()

    def done(self):
        self.elapsed = time.time() - self.start
        elapsed_disp = self.format_time(self.elapsed)
        if not self.hide:
            # Print completed bar with elapsed time
            STREAM.write(
                BAR_TEMPLATE
                % (
                    self.label,
                    self.filled_char * self.width,
                    self.empty_char * 0,
                    self.last_progress,
                    self.expected_size,
                    elapsed_disp,
                )
            )
            STREAM.write("\n")
            STREAM.flush()

    @staticmethod
    def format_time(seconds):
        return time.strftime("%H:%M:%S", time.gmtime(seconds))


def progress_bar(items, label="", width=32, hide=None, expected_size=None, every=1):
    """Progress iterator. Wrap your iterables with it."""

    count = len(items) if expected_size is None else expected_size

    with Bar(
        label=label,
        width=width,
        hide=hide,
        empty_char=BAR_EMPTY_CHAR,
        filled_char=BAR_FILLED_CHAR,
        expected_size=count,
        every=every,
    ) as _bar:
        for i, item in enumerate(items):
            yield item
            _bar.show(i + 1)


def create_progress_callback(encoder):
    encoder_len = encoder.len
    callback_bar = Bar(expected_size=encoder_len, filled_char="=")

    def callback(monitor):
        callback_bar.show(monitor.bytes_read)

    return callback, callback_bar



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import pytest

from polyaxon import settings
from polyaxon.stores.async_manager import (
    delete_dir,
    delete_file,
    download_dir,
    download_file,
)
from polyaxon.utils.path_utils import check_or_create_path
from tests.test_streams.base import create_tmp_files, set_store


@pytest.mark.asyncio
@pytest.mark.filterwarnings("ignore::RuntimeWarning")
async def test_download_dir_archive():
    store_root = set_store()
    path = os.path.join(store_root, "foo")
    check_or_create_path(path, is_dir=True)
    create_tmp_files(path)
    await download_dir(subpath="foo", to_tar=True)

    path_to = os.path.join(settings.AGENT_CONFIG.artifacts_root, "foo")
    assert os.path.exists(path_to)
    assert os.path.exists(path_to + "/0")
    assert os.path.exists(path_to + "/1")
    tar_path = os.path.join(settings.CLIENT_CONFIG.archive_root, "foo.tar.gz")
    assert os.path.exists(tar_path)


@pytest.mark.asyncio
@pytest.mark.filterwarnings("ignore::RuntimeWarning")
async def test_download_file():
    store_root = set_store()
    path = os.path.join(store_root, "foo")
    check_or_create_path(path, is_dir=True)
    create_tmp_files(path)
    await download_file(subpath="foo/0", check_cache=False)

    path_to = os.path.join(settings.AGENT_CONFIG.artifacts_root, "foo/0")
    assert os.path.exists(path_to)


@pytest.mark.asyncio
@pytest.mark.filterwarnings("ignore::RuntimeWarning")
async def test_delete_file():
    store_root = set_store()
    path = os.path.join(store_root, "foo")
    check_or_create_path(path, is_dir=True)
    create_tmp_files(path)
    filepath = "{}/0".format(path)
    assert os.path.exists(path) is True
    assert os.path.exists(filepath) is True
    await delete_file(subpath="foo/0")
    assert os.path.exists(path) is True
    assert os.path.exists(filepath) is False


@pytest.mark.asyncio
@pytest.mark.filterwarnings("ignore::RuntimeWarning")
async def test_delete_dir():
    store_root = set_store()
    path = os.path.join(store_root, "foo")
    check_or_create_path(path, is_dir=True)
    create_tmp_files(path)
    filepath = "{}/0".format(path)
    assert os.path.exists(path) is True
    assert os.path.exists(filepath) is True
    assert os.path.exists(filepath) is True
    await delete_dir(subpath="foo")
    assert os.path.exists(path) is False
    assert os.path.exists(filepath) is False



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# # coding: utf-8
#
#
# import requests
# import time
#
# from polyaxon import settings
# from tests.test_transports.utils import BaseTestCaseTransport
#
# from polyaxon.client.transport.periodic_transport import PeriodicHttpTransportMixin
# from polyaxon.client.workers.periodic_worker import PeriodicWorker
#
#
# class DummyTransport(PeriodicHttpTransportMixin):
#     # pylint:disable=protected-access
#     def __init__(self, delay=0):
#         self.queue = []
#         self.delay = delay
#         self._periodic_http_exceptions = 0
#         self._periodic_http_done = 0
#
#     def post(self, url, **kwargs):
#         time.sleep(self.delay)
#         self.queue.append(("post", url, kwargs))
#
#
# class ExceptionTransport(PeriodicHttpTransportMixin):
#     # pylint:disable=protected-access
#     def __init__(self, delay=0):
#         self.delay = delay
#         self._periodic_http_exceptions = 0
#         self._periodic_http_done = 0
#
#     def post(self, **kwargs):
#         time.sleep(self.delay)
#         raise requests.exceptions.HTTPError("error")
#
#
# class TestPeriodicTransport(BaseTestCaseTransport):
#     # pylint:disable=protected-access
#     def setUp(self):
#         super().setUp()
#         self.transport = DummyTransport()
#         self.exception_transport = ExceptionTransport()
#         settings.CLIENT_CONFIG.timeout = 0.01
#         settings.CLIENT_CONFIG.interval = 0
#
#     def test_retry_session(self):
#         assert hasattr(self.transport, "_retry_session") is False
#         assert isinstance(self.transport.retry_session, requests.Session)
#         assert isinstance(self.transport._retry_session, requests.Session)
#
#     def test_worker(self):
#         assert hasattr(self.transport, "_periodic_http_worker") is False
#         self.assertEqual(self.transport.periodic_http_worker, None)
#         worker = self.transport.get_periodic_http_worker(request=self.transport.post)
#         assert isinstance(self.transport.periodic_http_worker, PeriodicWorker)
#         assert isinstance(self.transport._periodic_http_worker, PeriodicWorker)
#         assert isinstance(worker, PeriodicWorker)
#
#     def test_periodic_requests_with_data(self):
#         assert self.transport.queue == []
#
#         self.transport.periodic_post(url="url_post", data={"d1": "v1"})
#         time.sleep(0.03)
#         queue = self.transport.queue
#         assert len(queue) == 1
#         assert queue[0][0] == "post"
#         assert queue[0][1] == "url_post"
#         assert queue[0][2]["data"] == [{"d1": "v1"}]
#         assert self.transport.periodic_http_done == 1
#         assert self.transport.periodic_http_exceptions == 0
#         assert self.transport.get_periodic_http_worker().is_alive() is True
#
#         self.transport.periodic_post(url="url_post", data={"d1": "v1"})
#         self.transport.periodic_post(url="url_post", data={"d2": "v2"})
#         time.sleep(0.03)
#         queue = self.transport.queue
#         assert len(queue) == 2
#         assert queue[1][0] == "post"
#         assert queue[1][1] == "url_post"
#         assert queue[1][2]["data"] == [{"d1": "v1"}, {"d2": "v2"}]
#         assert self.transport.periodic_http_done == 2
#         assert self.transport.periodic_http_exceptions == 0
#         assert self.transport.get_periodic_http_worker().is_alive() is True
#
#     def test_periodic_requests_with_json_data(self):
#         assert self.transport.queue == []
#
#         self.transport.periodic_post(url="url_post", json_data={"d1": "v1"})
#         time.sleep(0.03)
#         queue = self.transport.queue
#         assert len(queue) == 1
#         assert queue[0][0] == "post"
#         assert queue[0][1] == "url_post"
#         assert queue[0][2]["json_data"] == [{"d1": "v1"}]
#         assert self.transport.periodic_http_done == 1
#         assert self.transport.periodic_http_exceptions == 0
#         assert self.transport.get_periodic_http_worker().is_alive() is True
#
#         self.transport.periodic_post(url="url_post", json_data={"d1": "v1"})
#         self.transport.periodic_post(url="url_post", json_data={"d2": "v2"})
#         time.sleep(0.03)
#         queue = self.transport.queue
#         assert len(queue) == 2
#         assert queue[1][0] == "post"
#         assert queue[1][1] == "url_post"
#         assert queue[1][2]["json_data"] == [{"d1": "v1"}, {"d2": "v2"}]
#         assert self.transport.periodic_http_done == 2
#         assert self.transport.periodic_http_exceptions == 0
#         assert self.transport.get_periodic_http_worker().is_alive() is True
#
#     def test_periodic_requests_with_different_urls(self):
#         self.transport.periodic_post(url="url_post1", json_data={"d11": "v11"})
#         self.transport.periodic_post(url="url_post1", json_data={"d12": "v12"})
#         time.sleep(0.0001)
#         self.transport.periodic_post(url="url_post2", data={"d21": "v21"})
#         self.transport.periodic_post(url="url_post2", data={"d22": "v22"})
#         time.sleep(0.05)
#         queue = self.transport.queue
#         assert len(queue) == 2
#         assert queue[0][0] == "post"
#         assert queue[1][0] == "post"
#         assert queue[0][1] == "url_post1"
#         assert queue[1][1] == "url_post2"
#         assert queue[0][2]["json_data"] == [{"d11": "v11"}, {"d12": "v12"}]
#         assert queue[1][2]["data"] == [{"d21": "v21"}, {"d22": "v22"}]
#         assert self.transport.periodic_http_done == 2
#         assert self.transport.periodic_http_exceptions == 0
#         assert self.transport.periodic_http_worker.is_alive() is True
#
#     def test_async_exceptions(self):
#         self.exception_transport.periodic_post(url="url_post", data={"d1": "v1"})
#         time.sleep(0.03)
#         assert self.exception_transport.periodic_http_done == 1
#         assert self.exception_transport.periodic_http_exceptions == 1
#
#     def test_worker_atexit_handle_queue_before_stopping(self):
#         # Transport
#         settings.CLIENT_CONFIG.timeout = 0.5
#         self.transport.delay = 0.5
#         assert self.transport.queue == []
#         self.transport.periodic_post(url="url_post", data={"d1": "v1"})
#         assert self.transport.queue == []
#         time.sleep(0.03)
#         assert self.transport.get_periodic_http_worker().is_alive() is True
#         self.transport.get_periodic_http_worker().atexit()
#         queue = self.transport.queue
#         assert len(queue) == 1
#         assert queue[0][0] == "post"
#         assert queue[0][1] == "url_post"
#         assert queue[0][2]["data"] == [{"d1": "v1"}]
#         assert self.transport.periodic_http_done == 1
#         assert self.transport.periodic_http_exceptions == 0
#         assert self.transport.periodic_http_worker.is_alive() is False
#
#         # Exception transport
#         self.exception_transport.delay = 0.5
#         self.exception_transport.periodic_post(url="url_post", data={"d1": "v1"})
#         assert self.exception_transport.periodic_http_done == 0
#         assert self.exception_transport.periodic_http_exceptions == 0
#         time.sleep(0.1)
#         assert self.exception_transport.periodic_http_worker.is_alive() is True
#         self.exception_transport.periodic_http_worker.atexit()
#         assert self.exception_transport.periodic_http_done == 1
#         assert self.exception_transport.periodic_http_exceptions == 1
#         assert self.exception_transport.periodic_http_worker.is_alive() is False



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import pandas as pd
import pytest
import shutil

from polyaxon import settings
from polyaxon.polyboard.artifacts import V1ArtifactKind
from polyaxon.polyboard.events import V1Event, V1Events
from polyaxon.polyboard.events.schemas import LoggedEventListSpec
from polyaxon.streams.app.main import STREAMS_URL
from polyaxon.utils.path_utils import create_path
from tests.test_streams.base import get_streams_client, set_store
from tests.utils import BaseTestCase


@pytest.mark.streams_mark
class TestEventsEndpoints(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.store_root = set_store()
        self.run_path = os.path.join(self.store_root, "uuid")
        self.run_events = os.path.join(self.run_path, "events")
        # Create run artifacts path
        create_path(self.run_path)
        # Create run artifacts events path
        create_path(self.run_events)
        # Create run events
        self.create_tmp_events()

        self.client = get_streams_client()
        self.base_url = STREAMS_URL + "/namespace/owner/project/runs/uuid/events"

    def create_tmp_events(self):
        text1 = LoggedEventListSpec(
            name="text1",
            kind=V1ArtifactKind.TEXT,
            events=[
                V1Event.make(step=1, text="foo1"),
                V1Event.make(step=2, text="boo2"),
            ],
        )
        self.create_kind_events(name="text1", kind=V1ArtifactKind.TEXT, events=text1)
        text2 = LoggedEventListSpec(
            name="text2",
            kind=V1ArtifactKind.TEXT,
            events=[
                V1Event.make(step=1, text="foo2"),
                V1Event.make(step=2, text="boo2"),
            ],
        )
        self.create_kind_events(name="text2", kind=V1ArtifactKind.TEXT, events=text2)
        html1 = LoggedEventListSpec(
            name="html1",
            kind=V1ArtifactKind.HTML,
            events=[
                V1Event.make(step=1, html="foo1"),
                V1Event.make(step=2, html="boo2"),
            ],
        )
        self.create_kind_events(name="html1", kind=V1ArtifactKind.HTML, events=html1)
        html2 = LoggedEventListSpec(
            name="htm2",
            kind=V1ArtifactKind.HTML,
            events=[
                V1Event.make(step=1, html="foo2"),
                V1Event.make(step=2, html="boo2"),
            ],
        )
        self.create_kind_events(name="html2", kind=V1ArtifactKind.HTML, events=html2)

    def create_kind_events(self, name, kind, events):
        event_kind_path = "{}/{}".format(self.run_events, kind)
        create_path(event_kind_path)
        with open("{}/{}.plx".format(event_kind_path, name), "+w") as f:
            f.write(events.get_csv_header())
            f.write(events.get_csv_events())

    def test_download_events_cached(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True

        shutil.rmtree(self.run_events)
        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True

        shutil.rmtree(
            os.path.join(settings.CLIENT_CONFIG.archive_root, "uuid", "events")
        )
        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False

    def test_download_events_cached_force(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True

        shutil.rmtree(self.run_events)
        response = self.client.get(
            self.base_url + "/text?names=text1,text2&orient=csv&force=true"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False

    def test_download_text_events_as_csv(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/text?names=text1&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is False
        events1 = V1Events.read(name="text1", kind=V1ArtifactKind.TEXT, data=filepath1)
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        assert results[0].name == events1.name
        assert results[0].kind == events1.kind
        assert pd.DataFrame.equals(results[0].df, events1.df)

        response = self.client.get(self.base_url + "/text?names=text1,text2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        events2 = V1Events.read(name="text2", kind=V1ArtifactKind.TEXT, data=filepath2)
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url + "/text?names=text1,text2,text3&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

    def test_download_html_events_as_csv(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "html", "html1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "html", "html2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/html?names=html1&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is False
        events1 = V1Events.read(name="html1", kind=V1ArtifactKind.HTML, data=filepath1)
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        assert results[0].name == events1.name
        assert results[0].kind == events1.kind
        assert pd.DataFrame.equals(results[0].df, events1.df)

        response = self.client.get(self.base_url + "/html?names=html1,html2&orient=csv")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        events2 = V1Events.read(name="html2", kind=V1ArtifactKind.HTML, data=filepath2)
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url + "/html?names=text1,html1,html2&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        for res in response.json()["data"]:
            assert isinstance(res["data"], str)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

    def test_download_text_events_as_dict(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "text", "text2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/text?names=text1&orient=dict")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is False
        events1 = V1Events.read(
            name="text1", kind=V1ArtifactKind.TEXT, data=filepath1, parse_dates=False
        )
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        assert results[0].name == events1.name
        assert results[0].kind == events1.kind
        assert pd.DataFrame.equals(results[0].df, events1.df)

        response = self.client.get(
            self.base_url + "/text?names=text1,text2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        events2 = V1Events.read(
            name="text2", kind=V1ArtifactKind.TEXT, data=filepath2, parse_dates=False
        )
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url + "/text?names=text1,text2,text3&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

    def test_download_html_events_as_dict(self):
        filepath1 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "html", "html1.plx"
        )
        filepath2 = os.path.join(
            settings.CLIENT_CONFIG.archive_root, "uuid", "events", "html", "html2.plx"
        )
        assert os.path.exists(filepath1) is False
        assert os.path.exists(filepath2) is False
        response = self.client.get(self.base_url + "/html?names=html1&orient=dict")
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is False
        events1 = V1Events.read(
            name="html1", kind=V1ArtifactKind.HTML, data=filepath1, parse_dates=False
        )
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        assert results[0].name == events1.name
        assert results[0].kind == events1.kind
        assert pd.DataFrame.equals(results[0].df, events1.df)

        response = self.client.get(
            self.base_url + "/html?names=html1,html2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        events2 = V1Events.read(
            name="html2", kind=V1ArtifactKind.HTML, data=filepath2, parse_dates=False
        )
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url + "/html?names=text1,html1,html2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath1) is True
        assert os.path.exists(filepath2) is True
        for res in response.json()["data"]:
            assert isinstance(res["data"], dict)
        results = [V1Events.read(**i) for i in response.json()["data"]]
        expected = {events1.name: events1, events2.name: events2}
        for res in results:
            exp = expected[res.name]
            assert res.name == exp.name
            assert res.kind == exp.kind
            assert pd.DataFrame.equals(res.df, exp.df)


class TestMultiRunsEventsEndpoints(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.store_root = set_store()
        self.run1_path = os.path.join(self.store_root, "uuid1")
        self.run1_events = os.path.join(self.run1_path, "events")
        self.run2_path = os.path.join(self.store_root, "uuid2")
        self.run2_events = os.path.join(self.run2_path, "events")
        # Create run artifacts path
        create_path(self.run1_path)
        create_path(self.run2_path)
        # Create run artifacts events path
        create_path(self.run1_events)
        create_path(self.run2_events)
        # Create run events
        self.create_tmp_events(run_events=self.run1_events)
        self.create_tmp_events(run_events=self.run2_events)

        self.client = get_streams_client()
        self.base_url = STREAMS_URL + "/namespace/owner/project/runs/multi/events"

    def create_tmp_events(self, run_events: str):
        metric1 = LoggedEventListSpec(
            name="metric1",
            kind=V1ArtifactKind.METRIC,
            events=[
                V1Event.make(step=1, metric=1.1),
                V1Event.make(step=2, metric=1.2),
            ],
        )
        self.create_kind_events(
            run_events=run_events,
            name="metric1",
            kind=V1ArtifactKind.METRIC,
            events=metric1,
        )
        metric2 = LoggedEventListSpec(
            name="metric2",
            kind=V1ArtifactKind.METRIC,
            events=[
                V1Event.make(step=1, metric=2.1),
                V1Event.make(step=2, metric=2.2),
            ],
        )
        self.create_kind_events(
            run_events=run_events,
            name="metric2",
            kind=V1ArtifactKind.METRIC,
            events=metric2,
        )

    def create_kind_events(self, run_events, name, kind, events):
        event_kind_path = "{}/{}".format(run_events, kind)
        create_path(event_kind_path)
        with open("{}/{}.plx".format(event_kind_path, name), "+w") as f:
            f.write(events.get_csv_header())
            f.write(events.get_csv_events())

    def test_download_multi_metric_events_cached(self):
        filepath11 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath12 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric2.plx",
        )
        filepath21 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath22 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric2.plx",
        )

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True

        shutil.rmtree(self.run1_path)
        shutil.rmtree(self.run2_path)

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True

        shutil.rmtree(os.path.join(settings.CLIENT_CONFIG.archive_root, "uuid1"))
        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is False
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True

    def test_download_multi_metric_events_cached_force(self):
        filepath11 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath12 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric2.plx",
        )
        filepath21 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath22 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric2.plx",
        )

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True

        shutil.rmtree(self.run1_path)
        shutil.rmtree(self.run2_path)

        response = self.client.get(
            self.base_url
            + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict&force=true"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is False
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False

    def test_download_multi_metric_events_as_dict(self):
        filepath11 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath12 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric2.plx",
        )
        filepath21 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath22 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric2.plx",
        )
        assert os.path.exists(filepath11) is False
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False
        response = self.client.get(
            self.base_url + "/metric?names=metric1&runs=uuid1&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False
        events11 = V1Events.read(
            name="metric1",
            kind=V1ArtifactKind.METRIC,
            data=filepath11,
            parse_dates=False,
        )
        results = response.json()["data"]
        assert list(results.keys()) == ["uuid1"]
        assert len(results["uuid1"]) == 1
        results = [V1Events.read(**i) for i in response.json()["data"]["uuid1"]]
        assert results[0].name == events11.name
        assert results[0].kind == events11.kind
        assert pd.DataFrame.equals(results[0].df, events11.df)

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True
        events12 = V1Events.read(
            name="metric2",
            kind=V1ArtifactKind.METRIC,
            data=filepath12,
            parse_dates=False,
        )
        events21 = V1Events.read(
            name="metric1",
            kind=V1ArtifactKind.METRIC,
            data=filepath21,
            parse_dates=False,
        )
        events22 = V1Events.read(
            name="metric2",
            kind=V1ArtifactKind.METRIC,
            data=filepath22,
            parse_dates=False,
        )
        expected = {
            "uuid1": {events11.name: events11, events12.name: events12},
            "uuid2": {events21.name: events21, events22.name: events22},
        }
        response = response.json()["data"]
        results = {
            run_uuid: [V1Events.read(**i) for i in response[run_uuid]]
            for run_uuid in response
        }
        for run_uuid in results:
            for res in results[run_uuid]:
                exp = expected[run_uuid][res.name]
                assert res.name == exp.name
                assert res.kind == exp.kind
                assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url
            + "/metric?names=metric12,text2,text3&runs=uuid1,uuid2,uuid3&orient=dict"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True
        expected = {
            "uuid1": {events11.name: events11, events12.name: events12},
            "uuid2": {events21.name: events21, events22.name: events22},
        }
        response = response.json()["data"]
        results = {
            run_uuid: [V1Events.read(**i) for i in response[run_uuid]]
            for run_uuid in response
        }
        for run_uuid in results:
            for res in results[run_uuid]:
                exp = expected[run_uuid][res.name]
                assert res.name == exp.name
                assert res.kind == exp.kind
                assert pd.DataFrame.equals(res.df, exp.df)

    def test_download_multi_metric_events_as_csv(self):
        filepath11 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath12 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid1",
            "events",
            "metric",
            "metric2.plx",
        )
        filepath21 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric1.plx",
        )
        filepath22 = os.path.join(
            settings.CLIENT_CONFIG.archive_root,
            "uuid2",
            "events",
            "metric",
            "metric2.plx",
        )
        assert os.path.exists(filepath11) is False
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False
        response = self.client.get(
            self.base_url + "/metric?names=metric1&runs=uuid1&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is False
        assert os.path.exists(filepath21) is False
        assert os.path.exists(filepath22) is False
        events11 = V1Events.read(
            name="metric1", kind=V1ArtifactKind.METRIC, data=filepath11
        )
        results = response.json()["data"]
        assert list(results.keys()) == ["uuid1"]
        assert len(results["uuid1"]) == 1
        results = [V1Events.read(**i) for i in response.json()["data"]["uuid1"]]
        assert results[0].name == events11.name
        assert results[0].kind == events11.kind
        assert pd.DataFrame.equals(results[0].df, events11.df)

        response = self.client.get(
            self.base_url + "/metric?names=metric1,metric2&runs=uuid1,uuid2&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True
        events12 = V1Events.read(
            name="metric2", kind=V1ArtifactKind.METRIC, data=filepath12
        )
        events21 = V1Events.read(
            name="metric1", kind=V1ArtifactKind.METRIC, data=filepath21
        )
        events22 = V1Events.read(
            name="metric2", kind=V1ArtifactKind.METRIC, data=filepath22
        )
        expected = {
            "uuid1": {events11.name: events11, events12.name: events12},
            "uuid2": {events21.name: events21, events22.name: events22},
        }
        response = response.json()["data"]
        results = {
            run_uuid: [V1Events.read(**i) for i in response[run_uuid]]
            for run_uuid in response
        }
        for run_uuid in results:
            for res in results[run_uuid]:
                exp = expected[run_uuid][res.name]
                assert res.name == exp.name
                assert res.kind == exp.kind
                assert pd.DataFrame.equals(res.df, exp.df)

        response = self.client.get(
            self.base_url
            + "/metric?names=metric12,text2,text3&runs=uuid1,uuid2,uuid3&orient=csv"
        )
        assert response.status_code == 200
        assert os.path.exists(filepath11) is True
        assert os.path.exists(filepath12) is True
        assert os.path.exists(filepath21) is True
        assert os.path.exists(filepath22) is True
        expected = {
            "uuid1": {events11.name: events11, events12.name: events12},
            "uuid2": {events21.name: events21, events22.name: events22},
        }
        response = response.json()["data"]
        results = {
            run_uuid: [V1Events.read(**i) for i in response[run_uuid]]
            for run_uuid in response
        }
        for run_uuid in results:
            for res in results[run_uuid]:
                exp = expected[run_uuid][res.name]
                assert res.name == exp.name
                assert res.kind == exp.kind
                assert pd.DataFrame.equals(res.df, exp.df)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import pytest

from polyaxon import settings
from polyaxon.proxies.schemas.gateway.api import get_api_location_config
from polyaxon.proxies.schemas.gateway.auth import (
    get_auth_config,
    get_auth_location_config,
)
from polyaxon.proxies.schemas.gateway.dns import get_dns_config, get_resolver
from polyaxon.proxies.schemas.gateway.redirect import get_redirect_config
from polyaxon.proxies.schemas.gateway.services import (
    get_plugins_location_config,
    get_services_location_config,
)
from polyaxon.proxies.schemas.gateway.ssl import get_ssl_config
from polyaxon.proxies.schemas.gateway.streams import get_streams_location_config
from tests.utils import BaseTestCase


@pytest.mark.proxies_mark
class TestGatewaySchemas(BaseTestCase):
    SET_PROXIES_SETTINGS = True

    def test_ssl(self):
        expected = r"""
# SSL
ssl_session_timeout 1d;
ssl_session_cache shared:SSL:50m;
ssl_session_tickets off;

# modern configuration
ssl_protocols TLSv1.2;
ssl_ciphers ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256;
ssl_prefer_server_ciphers on;

# OCSP Stapling
ssl_stapling on;
ssl_stapling_verify on;
resolver 1.1.1.1 1.0.0.1 8.8.8.8 8.8.4.4 208.67.222.222 208.67.220.220 valid=60s;
resolver_timeout 2s;

ssl_certificate      /etc/ssl/polyaxon/polyaxon.com.crt;
ssl_certificate_key  /etc/ssl/polyaxon/polyaxon.com.key;
"""  # noqa
        assert get_ssl_config() == expected

        expected = r"""
# SSL
ssl_session_timeout 1d;
ssl_session_cache shared:SSL:50m;
ssl_session_tickets off;

# modern configuration
ssl_protocols TLSv1.2;
ssl_ciphers ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256;
ssl_prefer_server_ciphers on;

# OCSP Stapling
ssl_stapling on;
ssl_stapling_verify on;
resolver 1.1.1.1 1.0.0.1 8.8.8.8 8.8.4.4 208.67.222.222 208.67.220.220 valid=60s;
resolver_timeout 2s;

ssl_certificate      /foo/polyaxon.com.crt;
ssl_certificate_key  /foo/polyaxon.com.key;
"""  # noqa
        settings.PROXIES_CONFIG.ssl_path = "/foo"
        assert get_ssl_config() == expected

    def test_redirect_config(self):
        expected = r"""
server {
    listen 80;
    return 301 https://$host$request_uri;
}
"""  # noqa
        settings.PROXIES_CONFIG.ssl_enabled = False
        assert get_redirect_config() == ""
        settings.PROXIES_CONFIG.ssl_enabled = True
        assert get_redirect_config() == expected


@pytest.mark.proxies_mark
class TestGatewayServicesSchemas(BaseTestCase):
    SET_PROXIES_SETTINGS = True

    def test_service_dns_resolver(self):
        settings.PROXIES_CONFIG.auth_enabled = False
        expected = r"""
location ~ /services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    proxy_pass http://plx-operation-$4.$1.svc.cluster.local;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_use_resolver = False
        resolver = get_resolver()
        assert (
            get_services_location_config(resolver=resolver, auth="", rewrite=False)
            == expected
        )

        expected = r"""
location ~ /services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    proxy_pass http://plx-operation-$4.$1.svc.new-dns;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_prefix = "kube-dns.kube-system"
        settings.PROXIES_CONFIG.dns_use_resolver = True
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        resolver = get_resolver()
        assert (
            get_services_location_config(
                resolver=resolver, auth=get_auth_config(), rewrite=False
            )
            == expected
        )

    def test_services_dns_backend(self):
        settings.PROXIES_CONFIG.auth_enabled = False
        settings.PROXIES_CONFIG.dns_use_resolver = True
        expected = r"""
location ~ /services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.cluster.local valid=5s;
    proxy_pass http://plx-operation-$4.$1.svc.cluster.local;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_custom_cluster = "cluster.local"
        assert get_dns_config() == "kube-dns.kube-system.svc.cluster.local"
        resolver = get_resolver()
        assert (
            get_services_location_config(resolver=resolver, auth="", rewrite=False)
            == expected
        )

        expected = r"""
location ~ /services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    proxy_pass http://plx-operation-$4.$1.svc.new-dns;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        assert get_dns_config() == "kube-dns.kube-system.svc.new-dns"
        resolver = get_resolver()
        assert (
            get_services_location_config(
                resolver=resolver, auth=get_auth_config(), rewrite=False
            )
            == expected
        )

    def test_services_dns_prefix(self):
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_use_resolver = True
        expected = r"""
location ~ /services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver coredns.kube-system.svc.cluster.local valid=5s;
    proxy_pass http://plx-operation-$4.$1.svc.cluster.local;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_prefix = "coredns.kube-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "cluster.local"
        assert get_dns_config() == "coredns.kube-system.svc.cluster.local"
        resolver = get_resolver()
        assert (
            get_services_location_config(
                resolver=resolver, auth=get_auth_config(), rewrite=False
            )
            == expected
        )

        expected = r"""
location ~ /services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.new-system.svc.new-dns valid=5s;
    proxy_pass http://plx-operation-$4.$1.svc.new-dns;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_prefix = "kube-dns.new-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        assert get_dns_config() == "kube-dns.new-system.svc.new-dns"
        resolver = get_resolver()
        assert (
            get_services_location_config(
                resolver=resolver, auth=get_auth_config(), rewrite=False
            )
            == expected
        )


@pytest.mark.proxies_mark
class TestGatewayRewriteServicesSchemas(BaseTestCase):
    SET_PROXIES_SETTINGS = True

    def test_service_dns_resolver(self):
        settings.PROXIES_CONFIG.auth_enabled = False
        expected = r"""
location ~ /rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    rewrite_log on;
    rewrite ^/rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) /$5 break;
    proxy_pass http://plx-operation-$4.$1.svc.cluster.local;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_use_resolver = False
        resolver = get_resolver()
        assert (
            get_services_location_config(resolver=resolver, auth="", rewrite=True)
            == expected
        )

        expected = r"""
location ~ /rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) /$5 break;
    proxy_pass http://plx-operation-$4.$1.svc.new-dns;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_prefix = "kube-dns.kube-system"
        settings.PROXIES_CONFIG.dns_use_resolver = True
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        resolver = get_resolver()
        assert (
            get_services_location_config(
                resolver=resolver, auth=get_auth_config(), rewrite=True
            )
            == expected
        )

    def test_services_dns_backend(self):
        settings.PROXIES_CONFIG.auth_enabled = False
        settings.PROXIES_CONFIG.dns_use_resolver = True
        expected = r"""
location ~ /rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) /$5 break;
    proxy_pass http://plx-operation-$4.$1.svc.cluster.local;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_custom_cluster = "cluster.local"
        assert get_dns_config() == "kube-dns.kube-system.svc.cluster.local"
        resolver = get_resolver()
        assert (
            get_services_location_config(resolver=resolver, auth="", rewrite=True)
            == expected
        )

        expected = r"""
location ~ /rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) /$5 break;
    proxy_pass http://plx-operation-$4.$1.svc.new-dns;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        assert get_dns_config() == "kube-dns.kube-system.svc.new-dns"
        resolver = get_resolver()
        assert (
            get_services_location_config(
                resolver=resolver, auth=get_auth_config(), rewrite=True
            )
            == expected
        )

    def test_services_dns_prefix(self):
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_use_resolver = True
        expected = r"""
location ~ /rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver coredns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) /$5 break;
    proxy_pass http://plx-operation-$4.$1.svc.cluster.local;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_prefix = "coredns.kube-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "cluster.local"
        assert get_dns_config() == "coredns.kube-system.svc.cluster.local"
        resolver = get_resolver()
        assert (
            get_services_location_config(
                resolver=resolver, auth=get_auth_config(), rewrite=True
            )
            == expected
        )

        expected = r"""
location ~ /rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.new-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/rewrite-services/v1/([-_.:\w]+)/([-_.:\w]+)/([-_.:\w]+)/runs/([-_.:\w]+)/(.*) /$5 break;
    proxy_pass http://plx-operation-$4.$1.svc.new-dns;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_prefix = "kube-dns.new-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        assert get_dns_config() == "kube-dns.new-system.svc.new-dns"
        resolver = get_resolver()
        assert (
            get_services_location_config(
                resolver=resolver, auth=get_auth_config(), rewrite=True
            )
            == expected
        )


@pytest.mark.proxies_mark
class TestGatewayPluginsSchemas(BaseTestCase):
    SET_PROXIES_SETTINGS = True

    def test_no_plugins(self):
        assert get_plugins_location_config(resolver="", auth="") == []

    def test_plugins(self):
        proxy_services = {"tensorboard": {"port": 6006}, "notebook": {"port": 8888}}
        assert (
            len(
                get_plugins_location_config(
                    resolver="", auth="", proxy_services=proxy_services
                )
            )
            == 2
        )

    def test_plugins_dns_resolver(self):
        settings.PROXIES_CONFIG.auth_enabled = False
        proxy_services = {"tensorboard": {"port": 6006}, "notebook": {"port": 8888}}
        expected = r"""
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_use_resolver = False
        resolver = get_resolver()
        assert (
            "\n".join(
                get_plugins_location_config(
                    resolver=resolver, auth="", proxy_services=proxy_services
                )
            )
            == expected
        )

        expected = r"""
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_prefix = "kube-dns.kube-system"
        settings.PROXIES_CONFIG.dns_use_resolver = True
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        resolver = get_resolver()
        assert (
            "\n".join(
                get_plugins_location_config(
                    resolver=resolver,
                    auth=get_auth_config(),
                    proxy_services=proxy_services,
                )
            )
            == expected
        )

    def test_plugins_dns_backend(self):
        proxy_services = {"tensorboard": {"port": 6006}, "notebook": {"port": 8888}}
        settings.PROXIES_CONFIG.auth_enabled = False
        settings.PROXIES_CONFIG.dns_use_resolver = True
        expected = r"""
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_custom_cluster = "cluster.local"
        assert get_dns_config() == "kube-dns.kube-system.svc.cluster.local"
        resolver = get_resolver()
        assert (
            "\n".join(
                get_plugins_location_config(
                    resolver=resolver, auth="", proxy_services=proxy_services
                )
            )
            == expected
        )

        expected = r"""
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        assert get_dns_config() == "kube-dns.kube-system.svc.new-dns"
        resolver = get_resolver()
        assert (
            "\n".join(
                get_plugins_location_config(
                    resolver=resolver,
                    auth=get_auth_config(),
                    proxy_services=proxy_services,
                )
            )
            == expected
        )

    def test_plugins_dns_prefix(self):
        proxy_services = {"tensorboard": {"port": 6006}, "notebook": {"port": 8888}}
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_use_resolver = True
        expected = r"""
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver coredns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver coredns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_prefix = "coredns.kube-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "cluster.local"
        assert get_dns_config() == "coredns.kube-system.svc.cluster.local"
        resolver = get_resolver()
        assert (
            "\n".join(
                get_plugins_location_config(
                    resolver=resolver,
                    auth=get_auth_config(),
                    proxy_services=proxy_services,
                )
            )
            == expected
        )

        expected = r"""
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.new-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.new-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_prefix = "kube-dns.new-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        assert get_dns_config() == "kube-dns.new-system.svc.new-dns"
        resolver = get_resolver()
        assert (
            "\n".join(
                get_plugins_location_config(
                    resolver=resolver,
                    auth=get_auth_config(),
                    proxy_services=proxy_services,
                )
            )
            == expected
        )


@pytest.mark.proxies_mark
class TestGatewaySTreamsSchemas(BaseTestCase):
    SET_PROXIES_SETTINGS = True

    def test_streams_location_with_auth_config(self):
        expected = r"""
location /streams/ {
    proxy_pass http://polyaxon-polyaxon-streams;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa

        assert get_streams_location_config(resolver="", auth="") == expected

        settings.PROXIES_CONFIG.streams_port = 8888
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.streams_host = "foo"
        expected = r"""
location /streams/ {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    proxy_pass http://foo:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        assert (
            get_streams_location_config(resolver="", auth=get_auth_config()) == expected
        )

    def test_streams_location_with_dns_prefix(self):
        settings.PROXIES_CONFIG.auth_enabled = False
        settings.PROXIES_CONFIG.dns_use_resolver = True
        expected = r"""
location /streams/ {
    resolver coredns.kube-system.svc.cluster.local valid=5s;
    proxy_pass http://polyaxon-polyaxon-streams;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.dns_prefix = "coredns.kube-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "cluster.local"
        assert get_dns_config() == "coredns.kube-system.svc.cluster.local"
        resolver = get_resolver()
        assert get_streams_location_config(resolver=resolver, auth="") == expected

        expected = r"""
location /streams/ {
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;

    resolver kube-dns.new-system.svc.new-dns valid=5s;
    proxy_pass http://polyaxon-polyaxon-streams;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_buffering off;
}
"""  # noqa
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.dns_prefix = "kube-dns.new-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "new-dns"
        assert get_dns_config() == "kube-dns.new-system.svc.new-dns"
        resolver = get_resolver()
        assert (
            get_streams_location_config(resolver=resolver, auth=get_auth_config())
            == expected
        )


@pytest.mark.proxies_mark
class TestGatewayApiSchemas(BaseTestCase):
    SET_PROXIES_SETTINGS = True

    def test_api_location_config(self):
        expected = r"""
location = / {
    proxy_pass http://polyaxon-polyaxon-api;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}


location /api/v1/ {
    proxy_pass http://polyaxon-polyaxon-api;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}


location /ui/ {
    proxy_pass http://polyaxon-polyaxon-api;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}


location /sso/ {
    proxy_pass http://polyaxon-polyaxon-api;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}


location /static/ {
    proxy_pass http://polyaxon-polyaxon-api;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}
"""  # noqa
        assert get_api_location_config(resolver="") == expected

        settings.PROXIES_CONFIG.api_port = 8888
        settings.PROXIES_CONFIG.api_host = "foo"
        expected = r"""
location = / {
    proxy_pass http://foo:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}


location /api/v1/ {
    proxy_pass http://foo:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}


location /ui/ {
    proxy_pass http://foo:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}


location /sso/ {
    proxy_pass http://foo:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}


location /static/ {
    proxy_pass http://foo:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host $http_host;
    proxy_buffering off;
}
"""  # noqa
        assert get_api_location_config(resolver="") == expected

        settings.PROXIES_CONFIG.api_port = 443
        settings.PROXIES_CONFIG.api_host = "polyaxon.foo.com"
        expected = r"""
location = / {
    proxy_ssl_server_name on;
    proxy_pass https://polyaxon.foo.com;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host polyaxon.foo.com;
    proxy_buffering off;
}


location /api/v1/ {
    proxy_ssl_server_name on;
    proxy_pass https://polyaxon.foo.com;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host polyaxon.foo.com;
    proxy_buffering off;
}


location /ui/ {
    proxy_ssl_server_name on;
    proxy_pass https://polyaxon.foo.com;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host polyaxon.foo.com;
    proxy_buffering off;
}


location /sso/ {
    proxy_ssl_server_name on;
    proxy_pass https://polyaxon.foo.com;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host polyaxon.foo.com;
    proxy_buffering off;
}


location /static/ {
    proxy_ssl_server_name on;
    proxy_pass https://polyaxon.foo.com;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Host polyaxon.foo.com;
    proxy_buffering off;
}
"""  # noqa
        assert get_api_location_config(resolver="") == expected

    def test_auth_config(self):
        settings.PROXIES_CONFIG.auth_enabled = True
        expected = r"""
    auth_request     /auth/v1/;
    auth_request_set $auth_status $upstream_status;
"""  # noqa
        assert get_auth_config() == expected

        settings.PROXIES_CONFIG.auth_enabled = False
        assert get_auth_config() == ""

    def test_auth_location_config(self):
        settings.PROXIES_CONFIG.auth_use_resolver = False
        settings.PROXIES_CONFIG.dns_use_resolver = False
        settings.PROXIES_CONFIG.auth_enabled = True
        expected = r"""
location = /auth/v1/ {
    proxy_pass http://polyaxon-polyaxon-api;
    proxy_pass_request_body off;
    proxy_set_header Content-Length "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
    proxy_set_header X-Forwarded-Proto $scheme;
    proxy_set_header X-Origin-URI $request_uri;
    proxy_set_header X-Origin-Method $request_method;
    internal;
}
"""  # noqa
        assert get_auth_location_config(resolver="") == expected

        # Use resolver but do not enable it for auth
        settings.PROXIES_CONFIG.dns_use_resolver = True
        settings.PROXIES_CONFIG.dns_prefix = "coredns.kube-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "cluster.local"
        assert get_dns_config() == "coredns.kube-system.svc.cluster.local"
        resolver = get_resolver()

        expected = r"""
location = /auth/v1/ {
    proxy_pass http://polyaxon-polyaxon-api;
    proxy_pass_request_body off;
    proxy_set_header Content-Length "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
    proxy_set_header X-Forwarded-Proto $scheme;
    proxy_set_header X-Origin-URI $request_uri;
    proxy_set_header X-Origin-Method $request_method;
    internal;
}
"""  # noqa
        assert get_auth_location_config(resolver=resolver) == expected

        # Enable resolver for auth
        settings.PROXIES_CONFIG.auth_use_resolver = True
        expected = r"""
location = /auth/v1/ {
    resolver coredns.kube-system.svc.cluster.local valid=5s;
    proxy_pass http://polyaxon-polyaxon-api;
    proxy_pass_request_body off;
    proxy_set_header Content-Length "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
    proxy_set_header X-Forwarded-Proto $scheme;
    proxy_set_header X-Origin-URI $request_uri;
    proxy_set_header X-Origin-Method $request_method;
    internal;
}
"""  # noqa
        assert get_auth_location_config(resolver=resolver) == expected

    def test_external_auth_location_config(self):
        settings.PROXIES_CONFIG.auth_use_resolver = False
        settings.PROXIES_CONFIG.auth_enabled = True
        settings.PROXIES_CONFIG.auth_external = "https://cloud.polyaxon.com"
        expected = r"""
location = /auth/v1/ {
    proxy_ssl_server_name on;
    proxy_pass https://cloud.polyaxon.com;
    proxy_pass_request_body off;
    proxy_set_header Content-Length "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
    proxy_set_header X-Forwarded-Proto $scheme;
    proxy_set_header X-Origin-URI $request_uri;
    proxy_set_header X-Origin-Method $request_method;
    internal;
}
"""  # noqa
        assert get_auth_location_config(resolver="") == expected

        # Use resolver but do not enable it for auth
        settings.PROXIES_CONFIG.dns_use_resolver = True
        settings.PROXIES_CONFIG.dns_prefix = "coredns.kube-system"
        settings.PROXIES_CONFIG.dns_custom_cluster = "cluster.local"
        assert get_dns_config() == "coredns.kube-system.svc.cluster.local"
        resolver = get_resolver()

        expected = r"""
location = /auth/v1/ {
    proxy_ssl_server_name on;
    proxy_pass https://cloud.polyaxon.com;
    proxy_pass_request_body off;
    proxy_set_header Content-Length "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
    proxy_set_header X-Forwarded-Proto $scheme;
    proxy_set_header X-Origin-URI $request_uri;
    proxy_set_header X-Origin-Method $request_method;
    internal;
}
"""  # noqa
        assert get_auth_location_config(resolver=resolver) == expected



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from polyaxon.polyflow.plugins import V1Plugins
from tests.utils import BaseTestCase, assert_equal_dict


@pytest.mark.plugins_mark
class TestPluginsConfigs(BaseTestCase):
    def test_plugins_config(self):
        # Add auth
        config_dict = {"auth": True}

        config = V1Plugins.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add docker
        config_dict["docker"] = True
        config = V1Plugins.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add shm
        config_dict["shm"] = True
        config = V1Plugins.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add outputs
        config_dict["collectArtifacts"] = True
        config = V1Plugins.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add logs
        config_dict["collectLogs"] = True
        config = V1Plugins.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add resources
        config_dict["collectResources"] = True
        config = V1Plugins.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add notifications
        config_dict["notifications"] = [
            {"connections": ["test1"], "trigger": "succeeded"},
            {"connections": ["test2"], "trigger": "failed"},
            {"connections": ["test3"], "trigger": "done"},
        ]
        config = V1Plugins.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.connections.schemas import (
    V1BucketConnection,
    V1ClaimConnection,
    V1HostPathConnection,
    V1K8sResourceSchema,
)
from polyaxon.containers.contexts import CONTEXT_MOUNT_DOCKER
from polyaxon.k8s import k8s_schemas
from polyaxon.polypod.common import constants
from polyaxon.polypod.common.volumes import (
    get_artifacts_context_volume,
    get_configs_context_volume,
    get_connections_context_volume,
    get_docker_context_volume,
    get_shm_context_volume,
    get_volume,
    get_volume_from_config_map,
    get_volume_from_connection,
    get_volume_from_secret,
)
from polyaxon.schemas.types import V1ConnectionType, V1K8sResourceType
from tests.utils import BaseTestCase


@pytest.mark.polypod_mark
class TestVolumes(BaseTestCase):
    def test_get_volume_from_connection(self):
        # No store
        assert get_volume_from_connection(connection=None) is None

        # Bucket store
        store = V1ConnectionType(
            name="test",
            kind=V1ConnectionKind.S3,
            schema=V1BucketConnection(bucket="s3//:foo"),
        )
        assert get_volume_from_connection(connection=store) is None

        # Claim store
        store = V1ConnectionType(
            name="test",
            kind=V1ConnectionKind.VOLUME_CLAIM,
            schema=V1ClaimConnection(
                mount_path="/tmp", volume_claim="test", read_only=True
            ),
        )
        volume = get_volume_from_connection(connection=store)
        assert volume.name == store.name
        assert volume.persistent_volume_claim.claim_name == store.schema.volume_claim
        assert volume.persistent_volume_claim.read_only == store.schema.read_only

        # Host path
        store = V1ConnectionType(
            name="test",
            kind=V1ConnectionKind.HOST_PATH,
            schema=V1HostPathConnection(
                mount_path="/tmp", host_path="/tmp", read_only=True
            ),
        )
        volume = get_volume_from_connection(connection=store)
        assert volume.name == store.name
        assert volume.host_path == k8s_schemas.V1HostPathVolumeSource(
            path=store.schema.host_path
        )

    def test_get_volume_from_secret(self):
        # No store
        assert get_volume_from_secret(None) is None

        # Store with mount path
        resource1 = V1K8sResourceType(
            name="test1",
            schema=V1K8sResourceSchema(name="ref", items=["item1", "item2"]),
            is_requested=False,
        )
        assert get_volume_from_secret(resource1) is None

        # Claim store
        resource1 = V1K8sResourceType(
            name="test1",
            schema=V1K8sResourceSchema(
                name="ref", items=["item1", "item2"], mount_path="/tmp"
            ),
            is_requested=False,
        )
        volume = get_volume_from_secret(resource1)
        assert volume.name == resource1.name
        assert volume.secret.secret_name == resource1.name
        assert volume.secret.items == resource1.schema.items

    def test_get_volume_from_config_map(self):
        # No store
        assert get_volume_from_config_map(None) is None

        # Store with mount path
        resource1 = V1K8sResourceType(
            name="test1",
            schema=V1K8sResourceSchema(name="ref", items=["item1", "item2"]),
            is_requested=False,
        )
        assert get_volume_from_config_map(resource1) is None

        # Claim store
        resource1 = V1K8sResourceType(
            name="test1",
            schema=V1K8sResourceSchema(
                name="ref", items=["item1", "item2"], mount_path="/tmp"
            ),
            is_requested=False,
        )
        volume = get_volume_from_config_map(resource1)
        assert volume.name == resource1.name
        assert volume.config_map.name == resource1.name
        assert volume.config_map.items == resource1.schema.items

    def test_get_volume(self):
        # Empty dir
        volume = get_volume(volume="test")
        assert volume.name == "test"
        assert isinstance(volume.empty_dir, k8s_schemas.V1EmptyDirVolumeSource)

        # Claim
        volume = get_volume(volume="test", claim_name="claim_name", read_only=True)
        assert volume.name == "test"
        assert volume.persistent_volume_claim.claim_name == "claim_name"
        assert volume.persistent_volume_claim.read_only is True

        # Host path
        volume = get_volume(volume="test", host_path="/tmp", read_only=True)
        assert volume.name == "test"
        assert volume.host_path.path == "/tmp"

    def test_get_docker_context_volume(self):
        volume = get_docker_context_volume()
        assert volume.name == constants.CONTEXT_VOLUME_DOCKER
        assert volume.host_path.path == CONTEXT_MOUNT_DOCKER

    def test_get_configs_context_volume(self):
        volume = get_configs_context_volume()
        assert volume.name == constants.CONTEXT_VOLUME_CONFIGS
        assert isinstance(volume.empty_dir, k8s_schemas.V1EmptyDirVolumeSource)

    def test_get_artifacts_context_volume(self):
        volume = get_artifacts_context_volume()
        assert volume.name == constants.CONTEXT_VOLUME_ARTIFACTS
        assert isinstance(volume.empty_dir, k8s_schemas.V1EmptyDirVolumeSource)

    def test_get_connections_context_volume(self):
        volume = get_connections_context_volume("foo")
        assert volume.name == "foo"
        assert isinstance(volume.empty_dir, k8s_schemas.V1EmptyDirVolumeSource)

    def test_get_shm_context_volume(self):
        volume = get_shm_context_volume()
        assert volume.name == constants.CONTEXT_VOLUME_SHM
        assert isinstance(volume.empty_dir, k8s_schemas.V1EmptyDirVolumeSource)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

MAIN_JOB_CONTAINER = "polyaxon-main"
INIT_AUTH_CONTAINER = "polyaxon-init-auth"
SIDECAR_CONTAINER = "polyaxon-sidecar"
TFJOBS_CONTAINER = "tensorflow"
PYTORCHJOBS_CONTAINER = "pytorch"
INIT_DOCKERFILE_CONTAINER_PREFIX = "polyaxon-init-dockerfile"
INIT_FILE_CONTAINER_PREFIX = "polyaxon-init-file"
INIT_GIT_CONTAINER_PREFIX = "polyaxon-init-git"
INIT_CUSTOM_CONTAINER_PREFIX = "polyaxon-init-custom"
INIT_ARTIFACTS_CONTAINER_PREFIX = "polyaxon-init-artifacts"
POLYAXON_INIT_PREFIX = "polyaxon-init"
INIT_PREFIX = "init"
SIDECAR_PREFIX = "sidecar"


def generate_container_name(
    prefix: str, suffix: str = None, unique: bool = True
) -> str:
    import uuid

    prefix = prefix or "container"
    unique_prefix = uuid.uuid4().hex[:10]
    if suffix:
        if unique:
            suffix = "{}-{}".format(suffix, unique_prefix)
    else:
        suffix = unique_prefix
    return "{}-{}".format(prefix, suffix)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import Dict, List

from polyaxon.polyflow import V1CompiledOperation, V1Init
from polyaxon.schemas.types import V1ConnectionType
from polyaxon.utils.list_utils import to_list


class BaseContextsManager:
    @classmethod
    def _resolve_init_contexts(
        cls,
        contexts: Dict,
        init: List[V1Init],
        connection_by_names: Dict[str, V1ConnectionType],
    ):
        init = to_list(init, check_none=True)
        connections = [i.connection for i in init if i.connection]
        return cls._resolve_connections_contexts(
            contexts=contexts,
            connections=connections,
            connection_by_names=connection_by_names,
            key="init",
        )

    @staticmethod
    def _resolve_connections_contexts(
        contexts: Dict,
        connections: List[str],
        connection_by_names: Dict[str, V1ConnectionType],
        key: str = "connections",
    ) -> Dict:
        connections = to_list(connections, check_none=True)
        for connection in connections:
            if connection_by_names[connection].schema:
                contexts[key][connection] = connection_by_names[
                    connection
                ].schema.to_dict()
        return contexts

    @classmethod
    def _resolver_replica(
        cls,
        contexts: Dict,
        init: List[V1Init],
        connections: List[str],
        connection_by_names: Dict[str, V1ConnectionType],
    ) -> Dict:
        contexts["init"] = {}
        contexts["connections"] = {}
        contexts = cls._resolve_init_contexts(
            contexts=contexts, init=init, connection_by_names=connection_by_names
        )
        contexts = cls._resolve_connections_contexts(
            contexts=contexts,
            connections=connections,
            connection_by_names=connection_by_names,
        )
        return contexts

    @classmethod
    def resolve(
        cls,
        namespace: str,
        owner_name: str,
        project_name: str,
        run_uuid: str,
        contexts: Dict,
        compiled_operation: V1CompiledOperation,
        connection_by_names: Dict[str, V1ConnectionType],
    ) -> Dict:
        raise NotImplementedError()
